/*
 *  @Copyright (C) 2007 Kim Jin Chul (shriekout@gmail.com).
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*********************************************
 * 콜백 함수들
 * *******************************************/

#include <gtk/gtk.h>
#include <time.h>
#include <string.h>
#include <libintl.h>
#include "interface.h"
#include "happytimer.h"
#include "callback.h"
#include "timecheck.h"
#include "lib/libsexy/sexy-tooltip.h"

static gint event_10m = 0;
static gboolean cursor_enter_tooltip = FALSE;

void destroy(GtkWidget *widget, gpointer data)
{
	gtk_main_quit();
}

void show_timer(GtkWidget *widget, gpointer data)
{
	static gboolean window_shown = FALSE;

	if (!window_shown) {
		gtk_widget_show(timer);
	} else {
		gtk_widget_hide(timer);
	}

	window_shown = !window_shown;
	
	if (alarm_window != NULL) {
		alarm_window_destroy(NULL);
	}
}

gboolean delete_event(GtkWidget *widget,
		GdkEvent *event,
		gpointer data)
{
	show_timer(NULL, NULL);

	return TRUE;
}

void timerGo(GtkWidget *widget, gpointer data)
{
	GtkWidget *ta;
	GtkWidget *bi;
	GtkWidget *bl;
	GtkWidget *ev;
	GtkWidget *ss;
	GtkWidget *pw;
	char *buf;
	char buf2[80];
	long sec;
	struct tm *ptr;
	gdouble spin_sec;
	xmlDocPtr doc;
	xmlNodePtr root_node = NULL;
	xmlNodePtr ret = NULL;

	varList.countdown = !varList.countdown;

	if (varList.repeat && (strcmp((char *)data, "STOP"))) {
		varList.countdown = TRUE;
	}

	ta = g_object_get_data(G_OBJECT(timer), "timer_adj");
	bi = g_object_get_data(G_OBJECT(timer), "button_image");
	bl = g_object_get_data(G_OBJECT(timer), "button_label");
	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
	ss = g_object_get_data(G_OBJECT(ev), "sec_spin");

	if (varList.countdown) {
		gtk_image_set_from_file(GTK_IMAGE(trayiconImage), START_IMAGE);

		gtk_image_set_from_stock(GTK_IMAGE(bi), "gtk-media-stop",
				GTK_ICON_SIZE_BUTTON);
		gtk_label_set_text_with_mnemonic(GTK_LABEL(bl), _("Stop"));

		varList.timerCount = (time_t)gtk_adjustment_get_value(GTK_ADJUSTMENT(ta));
		ptr = gmtime(&varList.timerCount);

		buf = (char*)malloc(3 * sizeof(char));
		strftime(buf, 3, "%S", ptr);

		sec = atol(buf);
		varList.timerCount -= sec;
		free(buf);

		spin_sec = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ss));
		varList.timerCount += spin_sec;

		timerWidget_sensitive(FALSE);

		if ((doc = xmlReadFile(prefer.prefer_file, NULL, 0)) == NULL) {
			fprintf(stderr, "Document not parsed successfully\n");
			exit(1);
		} else {
			root_node = xmlDocGetRootElement(doc);
			ret = search_xml_name(root_node, "load_ptime");
			sprintf(buf2, "%ld", varList.timerCount);
			xmlSetProp(ret, BAD_CAST "ptime", BAD_CAST buf2);
			xmlFree(ret);
			xmlSaveFormatFileEnc(prefer.prefer_file, doc, "UTF-8", 1);
			xmlFreeDoc(doc);
		}
	} else {
		gtk_image_set_from_file(GTK_IMAGE(trayiconImage), TRAY_IMAGE);

		gtk_image_set_from_stock(GTK_IMAGE(bi), "gtk-media-play",
				GTK_ICON_SIZE_BUTTON);
		gtk_label_set_text_with_mnemonic(GTK_LABEL(bl), _("Start"));

		maxtimer_changed();

		timerWidget_sensitive(TRUE);

		if (cursor_enter_tooltip == FALSE)
			gtk_widget_hide(trayiconTooltip);

		if (event_10m > 0) {
			g_source_remove(event_10m);
			event_10m = 0;
		}
	}

	varList.is_pause = FALSE;
	pw = g_object_get_data(G_OBJECT(timer), "timerPause_togglebutton");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pw), FALSE);
}

void alarm_window_destroy(GtkWidget *widget)
{
	gst_element_set_state(GST_ELEMENT(gstSound[ARM_SND]), GST_STATE_READY);

	gtk_widget_destroy(alarm_window);
	alarm_window = NULL;
}

void show_preference(GtkWidget *widget, gpointer data)
{
	static GtkWidget *preference_window = NULL;
	struct soundDir *cur;

	if (!preference_window) {
		preference_window = create_preference();
		gtk_widget_show(preference_window);
	} else {
		if ((gboolean)data) {
			gtk_widget_destroy(preference_window);
			preference_window = NULL;

			cur = soundDir_head;

			while (cur != NULL) {
				soundDir_head =  cur->next;
				free(cur->dirname);
				free(cur);
				cur = soundDir_head;
			}

			soundDir_head = NULL;
		} else {
			gtk_window_present(GTK_WINDOW(preference_window));
		}
	}
}

void on_sound_checkbox_clicked(GtkWidget *widget, gpointer data)
{
	gboolean is_active;

	is_active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	
	if (is_active) {
		if (!strcmp((char *)data, "rtsound")) {
			free(prefer.rtsound);
			prefer.rtsound = strdup("on");
		} else if (!strcmp((char *)data, "tsound")) {
			free(prefer.tsound);
			prefer.tsound = strdup("on");
		} else if (!strcmp((char *)data, "load_ptime")) {
			free(prefer.load_ptime);
			prefer.load_ptime = strdup("on");
		}
	} else {
		if (!strcmp((char *)data, "rtsound")) {
			free(prefer.rtsound);
			prefer.rtsound = strdup("off");
		} else if (!strcmp((char *)data, "tsound")) {
			free(prefer.tsound);
			prefer.tsound = strdup("off");
		} else if (!strcmp((char *)data, "load_ptime")) {
			free(prefer.load_ptime);
			prefer.load_ptime = strdup("off");
		}
	}

	save_preference(widget, data);
}

void on_maxtimer_spin_changed(GtkWidget *widget, gpointer data)
{
	time_t max;
	char buf[80];

	max = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget)) * 60 * 60;

	sprintf(buf, "%ld\n", max);
	free(prefer.maxtimer);
	prefer.maxtimer = strdup(buf);

	if (!varList.countdown)
		maxtimer_changed();

	save_preference(NULL, "maxtimer");
}

void on_sound_combobox_changed(GtkWidget *widget, gpointer data)
{
	gint active;
	char buf[256];
	FILE *fp;
	int count;
	struct soundDir *cur;

	free(prefer.rtsound_dir);

	active = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

	cur = soundDir_head;
	for (count = 0; count < active; count++) {
		cur = cur->next;
	}

	prefer.rtsound_dir = strdup(cur->dirname);

	sprintf(buf, "%s%s", prefer.rtsound_dir, VOICE_INFO);

	if ((fp = fopen(buf, "r")) == NULL) {
		fprintf(stderr, "Error opening file %s.\n", buf);
		exit(1);
	}
	
	fgets(buf, 255, fp);
	fgets(buf, 255, fp);
	buf[strlen(buf) - 1] = '\0';
	
	free(varList.hour_clock);
	varList.hour_clock = strdup(buf);

	fgets(buf, 255, fp);
	buf[strlen(buf) - 1] = '\0';
	free(varList.sf);
	varList.sf = strdup(buf);

	fclose(fp);

	save_preference(NULL, "rtsound_dir");
}

void on_listenSound_button_clicked(GtkWidget *widget, gpointer data)
{
	char buf[256];

	sprintf(buf, "%s00h.%s", prefer.rtsound_dir, varList.sf);
	gst_element_set_state(gstSound[RT_SND], GST_STATE_READY);
	gchar *uri = g_filename_to_uri(buf, NULL, NULL);
	g_object_set(G_OBJECT(gstSound[RT_SND]), "uri", uri, NULL);
	gst_element_set_state(gstSound[RT_SND], GST_STATE_PLAYING);
	g_free(uri);
}

void on_image_button_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *imagechooserdialog;
	GdkPixbuf *pixbuf;
	GtkWidget *image;
	
	imagechooserdialog = create_imagechooserdialog();
	
	if (gtk_dialog_run(GTK_DIALOG(imagechooserdialog)) == GTK_RESPONSE_OK) {		
		free(prefer.timagefile);
		prefer.timagefile = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (imagechooserdialog));
		save_preference(NULL, "timagefile");
		
		pixbuf = gdk_pixbuf_new_from_file_at_scale(prefer.timagefile,
		-1, 100,
		TRUE,
		NULL);

		image = gtk_image_new_from_pixbuf(pixbuf);
		gtk_button_set_image(GTK_BUTTON(widget), image);
	}
	
	gtk_widget_destroy(imagechooserdialog);
}

void image_preview(GtkWidget *widget, gpointer data)
{
	GdkPixbuf *pixbuf;
	gchar *fn;
	
	fn = gtk_file_chooser_get_preview_filename(GTK_FILE_CHOOSER(widget));
	
	if (fn != NULL) {
		pixbuf = gdk_pixbuf_new_from_file_at_scale(fn,
			-1, 100,
			TRUE,
			NULL);

		if (!pixbuf) {
			gtk_image_set_from_icon_name(GTK_IMAGE(data), "gtk-missing-image", GTK_ICON_SIZE_DIALOG);
			gtk_image_set_pixel_size(GTK_IMAGE(data), 100);
		} else {
			gtk_image_set_from_pixbuf(GTK_IMAGE(data), pixbuf);
			gdk_pixbuf_unref(pixbuf);
		}
	}

	g_free(fn);
}

void sound_changed(GtkWidget *widget, gpointer data)
{
	gst_element_set_state(GST_ELEMENT(gstSound[PRE_SND]), GST_STATE_READY);

	/* if문으로 prefer.tsoundfile를 검사하는 이유는,
	이상하게도 이 부분이 실행될 때,
	여러번 실행되면서 쓰레기 값이 들어가고,
	프로그램이 죽어버림 ;;;*/
	if (prefer.tsoundfile) {
		free(prefer.tsoundfile);
	}

	prefer.tsoundfile = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));

	if (prefer.tsoundfile)
		save_preference(NULL, "tsoundfile");
}

void sound_preview(GtkWidget *widget, gpointer data)
{
	gchar *fn, *uri;

	fn = gtk_file_chooser_get_preview_filename(GTK_FILE_CHOOSER(widget));

	if (fn) {
		uri = g_filename_to_uri(fn, NULL, NULL);
		gst_element_set_state(GST_ELEMENT(gstSound[PRE_SND]), GST_STATE_READY);
		g_object_set(G_OBJECT(gstSound[PRE_SND]), "uri", uri, NULL);
		gst_element_set_state(gstSound[PRE_SND], GST_STATE_PLAYING);
		g_free(uri);
		g_free(fn);
	}
}

void on_minus_button_clicked(GtkWidget *widget)
{
	gdouble t;
	GtkWidget *message_dialog;
	
	t = gtk_adjustment_get_value(GTK_ADJUSTMENT(widget)) - 1800;

	if (t < 0) {
		message_dialog = gtk_message_dialog_new(NULL,
										GTK_DIALOG_MODAL,
										GTK_MESSAGE_ERROR,
										GTK_BUTTONS_OK,
										_("The value is too small.\nReturn to previous setting."));
		gtk_dialog_run(GTK_DIALOG(message_dialog));
		gtk_widget_destroy(message_dialog);

		return;
	}

	gtk_adjustment_set_value(GTK_ADJUSTMENT(widget), t);
}

void on_plus_button_clicked(GtkWidget *widget)
{
	gdouble t;
	time_t max;
	GtkWidget *message_dialog;
	GtkWidget *ev;
	GtkWidget *ms;

	t = gtk_adjustment_get_value(GTK_ADJUSTMENT(widget)) + 1800;
	max = atol(prefer.maxtimer);

	if (t > max) {
		message_dialog = gtk_message_dialog_new(NULL,
										GTK_DIALOG_MODAL,
										GTK_MESSAGE_ERROR,
										GTK_BUTTONS_OK,
										_("The value is too big.\nReturn to previous setting."));
		gtk_dialog_run(GTK_DIALOG(message_dialog));
		gtk_widget_destroy(message_dialog);

		return;
	}

	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
	ms = g_object_get_data(G_OBJECT(ev), "min_spin");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ms), 0);
	
	gtk_adjustment_set_value(GTK_ADJUSTMENT(widget), t);
}

void on_extension_button_clicked(GtkWidget *widget, gpointer data)
{
	static gboolean view_extension = TRUE;
	static gint width, height;
	GtkWidget *ev;

	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");

	if (view_extension) {
		gtk_image_set_from_stock(data, "gtk-go-up", GTK_ICON_SIZE_MENU);
		gtk_window_get_size(GTK_WINDOW(timer), &width, &height);
		gtk_widget_show(ev);
	} else {
		gtk_image_set_from_stock(data, "gtk-go-down", GTK_ICON_SIZE_MENU);
		gtk_widget_hide(ev);
		gtk_window_resize(GTK_WINDOW(timer), width, height);
	}

	view_extension = !view_extension;
}

void on_set_detail_spin_changed(GtkWidget *widget, gpointer data)
{
	GtkWidget *th;
	GtkWidget *ev;
	GtkWidget *hs;
	GtkWidget *ms;
	GtkWidget *ss;
	GtkWidget *tl;
	GtkWidget *ths;
	GtkWidget *message_dialog;
	gdouble spin_time;
	time_t max;
	char *buf, buf2[80];

	ths = g_object_get_data(G_OBJECT(timer), "timer_hscale");

	if (gtk_widget_is_focus(ths))
		return;

	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
	hs = g_object_get_data(G_OBJECT(ev), "hr_spin");
	ms = g_object_get_data(G_OBJECT(ev), "min_spin");
	ss = g_object_get_data(G_OBJECT(ev), "sec_spin");

	spin_time = gtk_spin_button_get_value(GTK_SPIN_BUTTON(hs)) * 3600;
	spin_time += gtk_spin_button_get_value(GTK_SPIN_BUTTON(ms)) * 60;
	spin_time += gtk_spin_button_get_value(GTK_SPIN_BUTTON(ss));

	max = atol(prefer.maxtimer);

	if (max < spin_time) {
		message_dialog = gtk_message_dialog_new(NULL,
										GTK_DIALOG_MODAL,
										GTK_MESSAGE_ERROR,
										GTK_BUTTONS_OK,
										_("The value is too big.\nReturn to previous setting."));
		gtk_dialog_run(GTK_DIALOG(message_dialog));
		gtk_widget_destroy(message_dialog);

		gtk_spin_button_set_value(GTK_SPIN_BUTTON(hs), varList.spin_hr);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(ms), varList.spin_min);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(ss), varList.spin_sec);
	} else {
		if (!strcmp((char *)data, "hour"))
			varList.spin_hr = gtk_spin_button_get_value(GTK_SPIN_BUTTON(hs));
		else if (!strcmp((char *)data, "minute"))
			varList.spin_min = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ms));
		else
			varList.spin_sec = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ss));

		th = g_object_get_data(G_OBJECT(timer), "timer_adj");
		gtk_adjustment_set_value(GTK_ADJUSTMENT(th), spin_time);

		buf = return_strftime(spin_time, TRUE);
		tl = g_object_get_data(G_OBJECT(timer), "timer_label");
		gtk_label_set_text(GTK_LABEL(tl), buf);

		sprintf(buf2, "Happytimer\n%s", buf);
		gtk_label_set_text(GTK_LABEL(tooltipTimer_label), _(buf2));

		free(buf);
	}
}

void on_repeatTimer_checkbox_clicked(GtkWidget *widget, gpointer data)
{
	varList.repeat = !varList.repeat;
}

void on_preset_combobox_changed(GtkWidget *widget, gpointer data)
{
	gint active;
	int count;
	GtkWidget *ev;
	GtkWidget *w;
	char buf[80];
	long t;
	struct presetList *cur;
	struct hms_struct tt;

	active = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

	cur = presetList_head;

	for (count = 0; count < active; count++)
		cur = cur->next;


	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");

	t = cur->timer;
	tt = return_hms_struct(t);

	w = g_object_get_data(G_OBJECT(ev), "sec_spin");
	sprintf(buf, "%d", tt.sec);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), atoi(buf));

	w = g_object_get_data(G_OBJECT(ev), "min_spin");
	sprintf(buf, "%d", tt.min);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), atoi(buf));

	w = g_object_get_data(G_OBJECT(ev), "hr_spin");
	sprintf(buf, "%d", tt.hr);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), atoi(buf));
}

static long return_timer(GtkWidget *w)
{
	GtkWidget *hw, *mw, *sw;
	long timer, hr, min, sec;

	hw = g_object_get_data(G_OBJECT(w), "hr_spin");
	mw = g_object_get_data(G_OBJECT(w), "min_spin");
	sw = g_object_get_data(G_OBJECT(w), "sec_spin");

	hr = gtk_spin_button_get_value(GTK_SPIN_BUTTON(hw)) * 3600;
	min = gtk_spin_button_get_value(GTK_SPIN_BUTTON(mw)) * 60;
	sec = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sw));

	timer = hr + min + sec;

	return timer;
}

void on_ppAdd_button_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *ew, *ev, *p;
	int result;
	struct presetList *new, *cur;;
	long time;
	char *buf1, buf2[80];

	dialog = create_presetEAdialog(P_ADD, NULL);

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	switch (result) {
		case GTK_RESPONSE_ACCEPT:
			ew = g_object_get_data(G_OBJECT(dialog), "entry");
			time = return_timer(dialog);

			new = (struct presetList *)malloc(sizeof(struct presetList));
			new->name = strdup(gtk_entry_get_text(GTK_ENTRY(ew)));
			new->timer = time;

			cur = presetList_head;

			while (cur->next != NULL) {
				cur = cur->next;
			}

			cur->next = new;
			new->next = NULL;

			buf1 = return_strftime(time, TRUE);
			sprintf(buf2, "%s (%s)", new->name, buf1);
			free(buf1);
			gtk_combo_box_append_text(GTK_COMBO_BOX(data), buf2);

			ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
			p = g_object_get_data(G_OBJECT(ev), "preset_combobox");
			gtk_combo_box_append_text(GTK_COMBO_BOX(p), buf2);

			ppREA(new, NULL, P_ADD);
			break;
		default:
			break;
	}

	gtk_widget_destroy(dialog);
}

void on_ppEdit_button_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	int active1, active2, count, result;
	struct presetList *cur, *past;
	GtkWidget *ew, *ev, *pb;
	time_t t;
	char *buf1, buf2[80];

	active1 = gtk_combo_box_get_active(GTK_COMBO_BOX(data));

	if (active1 == -1)
		return;

	cur = presetList_head->next;

	for (count = 0; count < active1; count++)
		cur = cur->next;

	dialog = create_presetEAdialog(P_EDIT, cur);

	result = gtk_dialog_run(GTK_DIALOG(dialog));

	switch (result) {
		case GTK_RESPONSE_ACCEPT:
			past = (struct presetList *)malloc(sizeof(struct presetList));
			past->name = strdup(cur->name);
			past->timer = cur->timer;

			ew = g_object_get_data(G_OBJECT(dialog), "entry");
			free(cur->name);
			cur->name = strdup(gtk_entry_get_text(GTK_ENTRY(ew)));

			t = return_timer(dialog);
			cur->timer = t;

			ppREA(past, cur, P_EDIT);

			buf1 = return_strftime(cur->timer, TRUE);

			sprintf(buf2, "%s (%s)", cur->name, buf1);
			free(buf1);

			gtk_combo_box_remove_text(GTK_COMBO_BOX(data), active1);
			gtk_combo_box_insert_text(GTK_COMBO_BOX(data), active1, buf2);
			gtk_combo_box_set_active(GTK_COMBO_BOX(data), active1);

			ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
			pb = g_object_get_data(G_OBJECT(ev), "preset_combobox");

			active2 = gtk_combo_box_get_active(GTK_COMBO_BOX(pb));
			gtk_combo_box_remove_text(GTK_COMBO_BOX(pb), active1 + 1);
			gtk_combo_box_insert_text(GTK_COMBO_BOX(pb), active1 + 1, buf2);

			if ((active1 + 1) == active2)
				gtk_combo_box_set_active(GTK_COMBO_BOX(pb), active2);
			break;
		default:
			break;
	}

	gtk_widget_destroy(dialog);
}

void on_ppRemove_button_clicked(GtkWidget *widget, gpointer data)
{
	int active1, active2, count, result;
	GtkWidget *dialog, *ev, *p;
	char *buf1, buf2[80];
	struct presetList *cur, *del;

	active1 = gtk_combo_box_get_active(GTK_COMBO_BOX(data));

	if (active1 == -1)
		return;

	cur = presetList_head;
	del = cur->next;

	for (count =0; count < active1; count++) {
		cur = del;
		del = del->next;
	}

	buf1 = return_strftime(del->timer, TRUE);

	sprintf(buf2, "%s (%s)", del->name, buf1);
	free(buf1);

	dialog = create_presetremoveDialog(buf2);
	result = gtk_dialog_run(GTK_DIALOG(dialog));

	switch (result) {
		case GTK_RESPONSE_OK:
			ppREA(del, NULL, P_REMOVE);
			cur->next = del->next;
			free(del->name);
			free(del);
			gtk_combo_box_remove_text(GTK_COMBO_BOX(data), active1);
			gtk_combo_box_set_active(GTK_COMBO_BOX(data), 0);

			ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
			p = g_object_get_data(G_OBJECT(ev), "preset_combobox");

			active2 = gtk_combo_box_get_active(GTK_COMBO_BOX(p));
			gtk_combo_box_remove_text(GTK_COMBO_BOX(p), active1+1);

			if (active2 == (active1 + 1))
				gtk_combo_box_set_active(GTK_COMBO_BOX(p), 0);

			break;
		default:
			break;
	}

	gtk_widget_destroy(dialog);
}

/* 이 함수는 리듬박스연주기(Rhythmbox)에서 GPL 라이센스 하에 C&P 한 함 수 입니다. */
void on_trayiconEbox_enter_notify_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	GdkScreen *screen;
	gint x, y;
	GdkRectangle rect;

	screen = gtk_widget_get_screen(widget);
	gdk_window_get_origin(widget->window, &x, &y);

	rect.x = widget->allocation.x + x;
	rect.y = widget->allocation.y + y;
	rect.width  = widget->allocation.width;
	rect.height = widget->allocation.height;

	sexy_tooltip_position_to_rect(SEXY_TOOLTIP(trayiconTooltip), &rect, screen);

	gtk_widget_show(trayiconTooltip);

	if (data) {
		if (!strcmp((char *)data, "EVENT")) {
			event_10m = g_timeout_add_seconds(5, (GtkFunction)on_trayiconEbox_leave_notify_event, NULL);
		}
	} else {
		if (event_10m > 0) {
			g_source_remove(event_10m);
			event_10m = 0;
		}

		cursor_enter_tooltip = TRUE;
	}
}

void on_trayiconEbox_leave_notify_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gtk_widget_hide(trayiconTooltip);

	cursor_enter_tooltip = FALSE;
}

void on_trayiconEbox_button_press_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	GdkEventButton *bevent = (GdkEventButton *) event;

	if (event->type != GDK_BUTTON_PRESS)
		return;

	switch (bevent->button) {
		case 1:	show_timer(NULL, NULL);
				break;
		case 3:	gtk_menu_popup(GTK_MENU(menu), NULL,
							NULL, NULL, NULL,
							bevent->button, bevent->time);
				break;
		default:
			break;
	}

}

void on_timerPause_button_toggled(GtkWidget *widget, gpointer data)
{
	gboolean toggled;

	toggled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	if (toggled) {
		varList.is_pause = TRUE;
	} else {
		varList.is_pause = FALSE;
	}
}
