/*
 *  @Copyright (C) 2007 Kim Jin Chul (shriekout@gmail.com).
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************
 * GUI를 구성합니다.
 * *************************************************/

#include <gtk/gtk.h>
#include <dirent.h>
#include <string.h>
#include <libintl.h>
#include "callback.h"
#include "happytimer.h"
#include "timecheck.h"
#include "interface.h"

static void display_about_dialog(void);

/* status 아이콘에서 오른쪽 버튼 클릭시 나타날 메뉴 */
GtkWidget *create_menu(void)
{
	GtkWidget *menu;
	GtkWidget *timer_menu;
	GdkPixbuf *pixbuf;
	GtkWidget *image;
	GtkWidget *timer_preference;
	GtkWidget *timer_about;
	GtkWidget *separator_menu;
	GtkWidget *quit_menu;
	GtkAccelGroup *accel_group;
	gint x, y;

	accel_group = gtk_accel_group_new();

	menu = gtk_menu_new();

	timer_menu = gtk_image_menu_item_new_with_label(_("timer"));
	gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &x, &y);
	pixbuf = gdk_pixbuf_new_from_file_at_size(HAPPYTIMER_ICON, x, y, NULL);
	image = gtk_image_new_from_pixbuf(pixbuf);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(timer_menu), image);
	gdk_pixbuf_unref(pixbuf);

	gtk_widget_show(image);
	gtk_widget_show(timer_menu);
	gtk_container_add(GTK_CONTAINER(menu), timer_menu);
	g_signal_connect(G_OBJECT(timer_menu), "activate",
			G_CALLBACK(show_timer), NULL);

	timer_preference = gtk_image_menu_item_new_from_stock("gtk-preferences", accel_group);
	gtk_container_add(GTK_CONTAINER(menu), timer_preference);
	gtk_widget_show(timer_preference);
	g_signal_connect(G_OBJECT(timer_preference), "activate",
			G_CALLBACK(show_preference), (gpointer)FALSE);

	timer_about = gtk_image_menu_item_new_from_stock("gtk-about", accel_group);
	gtk_widget_show(timer_about);
	gtk_container_add(GTK_CONTAINER(menu), timer_about);
	g_signal_connect(G_OBJECT(timer_about), "activate",
			G_CALLBACK(display_about_dialog), NULL);

	separator_menu = gtk_separator_menu_item_new();
	gtk_widget_show(separator_menu);
	gtk_container_add(GTK_CONTAINER(menu), separator_menu);
	gtk_widget_set_sensitive(separator_menu, TRUE);

	quit_menu = gtk_image_menu_item_new_from_stock("gtk-quit", accel_group);
	gtk_widget_show(quit_menu);
	gtk_container_add(GTK_CONTAINER(menu), quit_menu);
	g_signal_connect(G_OBJECT(quit_menu), "activate",
			G_CALLBACK(destroy), NULL);

	gtk_menu_set_accel_group(GTK_MENU(menu), accel_group);

	return menu;
}

/* 타이머 설정창 생성 */
GtkWidget *create_timer(void)
{
	GtkWidget *timer_window;
	GdkPixbuf *timer_window_icon_pixbuf;
	GtkWidget *basic_vbox;
	GtkWidget *forBorder_table;
	GtkWidget *border_eventbox;
	GdkColor color;
	GtkWidget *hbox;
	GtkWidget *hbox2;
	GtkWidget *vbox;
	GtkWidget *vbox2;
	GtkWidget *slide_hbox;
	GtkWidget *minus_button;
	GtkWidget *plus_button;
	GtkWidget *image1;
	GtkWidget *timer_hscale;
	GtkWidget *timer_label;
	GtkWidget *max_label;
	GtkWidget *timerPause_togglebutton;
	GtkWidget *toggle_image_pause;
	GtkWidget *timerStart_button;
	GtkWidget *alignment;
	GtkWidget *button_image;
	GtkWidget *button_label;
	GtkAdjustment *timer_adj;
	GtkWidget *extension_button;
	GtkWidget *extension_image;
	GtkWidget *extension_vbox;
	time_t max;
	guint x, y;
	char buf[10];

	timer_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(timer_window), GTK_WIN_POS_MOUSE);
	gtk_widget_set_size_request(timer_window, 450, -1);
	gtk_window_set_title(GTK_WINDOW(timer_window), _("happytimer"));
	timer_window_icon_pixbuf = gdk_pixbuf_new_from_file(HAPPYTIMER_ICON, NULL);
	gtk_window_set_icon(GTK_WINDOW(timer_window), timer_window_icon_pixbuf);
	gtk_window_set_decorated(GTK_WINDOW(timer_window), FALSE);
	gtk_window_stick(GTK_WINDOW(timer_window));
	gtk_window_set_keep_above(GTK_WINDOW(timer_window), TRUE);
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(timer_window), TRUE);
	gdk_pixbuf_unref(timer_window_icon_pixbuf);
	g_signal_connect(G_OBJECT(timer_window), "delete_event",
			G_CALLBACK(delete_event), NULL);

	forBorder_table = gtk_table_new(3, 3, FALSE);
	gtk_widget_show(forBorder_table);
	gtk_container_add(GTK_CONTAINER(timer_window), forBorder_table);

	/* 테두리 작업 시작 */
	gdk_color_parse("black", &color);

	for (y = 0; y < 3; y++)
		for (x = 0; x < 3; x++) {
			if ((x == 1) && (y == 1))
				continue;
			border_eventbox = gtk_event_box_new();
			gtk_widget_modify_bg(border_eventbox, GTK_STATE_NORMAL, &color);

			if (((y == 0) || (y == 2)) && (x == 1)) {
				gtk_table_attach(GTK_TABLE(forBorder_table), border_eventbox,
						x, x+1, y, y+1,
						(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
						(GtkAttachOptions)(0), 0, 0);
				gtk_widget_set_size_request(border_eventbox, -1, 2);
			} else if ((y == 1) && ((x == 0) || (x == 2))) {
				gtk_table_attach(GTK_TABLE(forBorder_table), border_eventbox,
						x, x+1, y, y+1,
						(GtkAttachOptions)(0),
						(GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
				gtk_widget_set_size_request(border_eventbox, 2, -1);
			} else {
				gtk_table_attach(GTK_TABLE(forBorder_table), border_eventbox,
						x, x+1, y, y+1,
						(GtkAttachOptions)(0),
						(GtkAttachOptions)(0), 0, 0);
				gtk_widget_set_size_request(border_eventbox, 2, 2);
			}

			gtk_widget_show(border_eventbox);
	}
	/* 테두리 작업 끝 */

	basic_vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(basic_vbox);
	gtk_table_attach(GTK_TABLE(forBorder_table), basic_vbox, 1, 2, 1, 2,
			(GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
			(GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 5, 5);

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(basic_vbox), hbox, TRUE, TRUE, 0);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
	
	slide_hbox = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(slide_hbox);
	gtk_box_pack_start(GTK_BOX(vbox), slide_hbox, TRUE, TRUE, 0);

	max = atol(prefer.maxtimer);
	timer_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, max, 60, 0, 0);
	g_object_set_data(G_OBJECT(timer_window), "timer_adj", (gpointer)timer_adj);
	g_signal_connect(G_OBJECT(timer_adj), "value_changed",
			G_CALLBACK(set_timer), NULL);

	minus_button = gtk_button_new();
	gtk_widget_set_tooltip_text(minus_button, _("-30min"));
	gtk_widget_show(minus_button);
	gtk_box_pack_start(GTK_BOX(slide_hbox), minus_button, FALSE, TRUE, 3);
	g_signal_connect_swapped(G_OBJECT(minus_button), "clicked",
			G_CALLBACK(on_minus_button_clicked), G_OBJECT(timer_adj));
	g_object_set_data(G_OBJECT(timer_window), "minus_button", (gpointer)minus_button);

	image1 = gtk_image_new_from_stock("gtk-go-back", GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(GTK_BUTTON(minus_button), image1);

	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox2);
	gtk_box_pack_start(GTK_BOX(slide_hbox), vbox2, TRUE, TRUE, 3);

	timer_hscale = gtk_hscale_new(timer_adj);
	gtk_widget_show(timer_hscale);
	gtk_box_pack_start(GTK_BOX(vbox2), timer_hscale, TRUE, TRUE, 3);
	gtk_scale_set_draw_value(GTK_SCALE(timer_hscale), FALSE);
	g_object_set_data(G_OBJECT(timer_window), "timer_hscale", (gpointer)timer_hscale);

	hbox2 = gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox2);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox2, TRUE, TRUE, 0);

	max_label = gtk_label_new("0");
	gtk_widget_show(max_label);
	gtk_box_pack_start(GTK_BOX(hbox2), max_label, FALSE, FALSE, 0);

	sprintf(buf, _("%ldhours"), max/3600);
	max_label = gtk_label_new(buf);
	gtk_widget_show(max_label);
	gtk_box_pack_end(GTK_BOX(hbox2), max_label, FALSE, FALSE, 0);
	g_object_set_data(G_OBJECT(timer_window), "max_label", (gpointer)max_label);

	plus_button = gtk_button_new();
	gtk_widget_set_tooltip_text(plus_button, _("+30min"));
	gtk_widget_show(plus_button);
	gtk_box_pack_start(GTK_BOX(slide_hbox), plus_button, FALSE, TRUE, 3);
	g_signal_connect_swapped(G_OBJECT(plus_button), "clicked",
			G_CALLBACK(on_plus_button_clicked), G_OBJECT(timer_adj));
	g_object_set_data(G_OBJECT(timer_window), "plus_button", (gpointer)plus_button);

	image1 = gtk_image_new_from_stock("gtk-go-forward", GTK_ICON_SIZE_BUTTON);
	gtk_button_set_image(GTK_BUTTON(plus_button), image1);

	timer_label = gtk_label_new(_("00hr 00min 00sec"));
	gtk_widget_show(timer_label);
	gtk_box_pack_start(GTK_BOX(vbox), timer_label, FALSE, FALSE, 0);
	gtk_label_set_justify(GTK_LABEL(timer_label), GTK_JUSTIFY_CENTER);
	g_object_set_data(G_OBJECT(timer_window), "timer_label", (gpointer)timer_label);

	timerPause_togglebutton = gtk_toggle_button_new();
	gtk_widget_set_tooltip_text(timerPause_togglebutton, _("Pause"));
	gtk_widget_show(timerPause_togglebutton);
	gtk_box_pack_start(GTK_BOX(hbox), timerPause_togglebutton, FALSE, TRUE, 0);
	g_signal_connect(G_OBJECT(timerPause_togglebutton), "toggled",
			G_CALLBACK(on_timerPause_button_toggled), NULL);
	g_object_set_data(G_OBJECT(timer_window), "timerPause_togglebutton", (gpointer)timerPause_togglebutton);

	toggle_image_pause = gtk_image_new_from_stock("gtk-media-pause",
					GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(toggle_image_pause);
	gtk_container_add(GTK_CONTAINER(timerPause_togglebutton), toggle_image_pause);

	timerStart_button = gtk_button_new();
	gtk_widget_show(timerStart_button);
	gtk_box_pack_start(GTK_BOX(hbox), timerStart_button, FALSE, TRUE, 0);
	g_signal_connect(G_OBJECT(timerStart_button), "clicked",
			G_CALLBACK(timerGo), (gpointer)"STOP");

	alignment = gtk_alignment_new(0.5, 0.5, 0, 0);
	gtk_widget_show(alignment);
	gtk_container_add(GTK_CONTAINER(timerStart_button), alignment);

	hbox2 = gtk_hbox_new(FALSE, 2);
	gtk_widget_show(hbox2);
	gtk_container_add(GTK_CONTAINER(alignment), hbox2);

	button_image = gtk_image_new_from_stock("gtk-media-play",
			GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(button_image);
	gtk_box_pack_start(GTK_BOX(hbox2), button_image, FALSE, FALSE, 0);
	g_object_set_data(G_OBJECT(timer_window), "button_image", (gpointer)button_image);

	button_label = gtk_label_new(_("Start"));
	gtk_widget_show(button_label);
	gtk_box_pack_start(GTK_BOX(hbox2), button_label, FALSE, FALSE, 0);
	g_object_set_data(G_OBJECT(timer_window), "button_label", (gpointer)button_label);

	extension_button = gtk_button_new();
	gtk_widget_set_tooltip_text(extension_button, _("expansion"));
	gtk_widget_show(extension_button);
	gtk_box_pack_start(GTK_BOX(basic_vbox), extension_button, TRUE, TRUE, 0);

	extension_image = gtk_image_new_from_stock("gtk-go-down", GTK_ICON_SIZE_MENU);
	gtk_widget_show(extension_image);
	gtk_container_add(GTK_CONTAINER(extension_button), extension_image);

	g_signal_connect(G_OBJECT(extension_button), "clicked",
			G_CALLBACK(on_extension_button_clicked), (gpointer)extension_image);

	extension_vbox = create_extension_box();
	gtk_box_pack_start(GTK_BOX(basic_vbox), extension_vbox, TRUE, TRUE, 0);
	g_object_set_data(G_OBJECT(timer_window), "extension_vbox", (gpointer)extension_vbox);

	return timer_window;
}


/* 타이머 종료시 나타날 창 */
GtkWidget *create_alarm_window(void)
{
	GtkWidget *window, *pixmap, *eventbox;
	GdkPixbuf *gdk_pixbuf;
	GdkPixmap *gdk_pixmap;
	GdkBitmap *mask;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ALWAYS);
	gtk_window_set_modal(GTK_WINDOW(window), TRUE);
	gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
	gtk_window_stick(GTK_WINDOW(window));
	gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), TRUE);
	gtk_window_present(GTK_WINDOW(window));
	gtk_widget_show(window);

	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(window), eventbox);
	gtk_widget_show(eventbox);
	g_signal_connect(G_OBJECT(eventbox), "button_press_event",
			G_CALLBACK(alarm_window_destroy), NULL);

	gdk_pixbuf = gdk_pixbuf_new_from_file(prefer.timagefile, NULL);
	gdk_pixbuf_render_pixmap_and_mask(gdk_pixbuf, &gdk_pixmap, &mask, 100);
	pixmap = gtk_pixmap_new(gdk_pixmap, mask);
	gdk_pixbuf_unref(gdk_pixbuf);
	gtk_container_add(GTK_CONTAINER(eventbox), pixmap);
	gtk_widget_show(pixmap);

	gtk_widget_shape_combine_mask(window, mask, 0, 0);

	return window;
}

/* 환경 설정창 생성 */
GtkWidget *create_preference(void)
{
	GtkWidget *window;
	GtkWidget *notebook;
	GtkWidget *label;
	GdkPixbuf *icon;
	GtkWidget *basic_vbox;
	GtkWidget *vbox;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *rTSound_checkbox;
	GtkWidget *tSound_checkbox;
	GtkWidget *pTimer_checkbox;
	GtkWidget *frame;
	GtkAdjustment *maxtimer_spin_adj;
	GtkWidget *maxtimer_spin;
	GtkFileFilter *audio_filter, *all_filter;
/*	GtkWidget *sound_chooser;*/
	GtkWidget *sound_combobox;
	GtkWidget *listenSound_button;
	GtkWidget *image_button;
	GdkPixbuf *pixbuf;
	GtkWidget *pp_combobox;
	GtkWidget *ppAdd_button;
	GtkWidget *ppEdit_button;
	GtkWidget *ppRemove_button;
	GtkWidget *image;
	GtkWidget *hseparator;
	GtkWidget *hbuttonbox;
	GtkWidget *closeButton;
	char *audio_mime_types[] = {
		"audio/3gpp",
		"audio/x-mp3",
		"audio/x-mpeg",
		"audio/mpeg",
		"audio/x-wav",
		"audio/x-mpegurl",
		"audio/mpegurl",
		"audio/x-scpls",
		"audio/x-m4a",
		"audio/x-ms-asf",
		"audio/x-ms-asx",
		"audio/x-ms-wax",
		"audio/x-real-audio",
		"audio/x-pn-realaudio",
		"application/x-flac",
		"audio/x-flac",
		"audio/vnd.rn-realaudio",
		"audio/x-pn-aiff",
		"audio/x-pn-au",
		"audio/x-pn-wav",
		"audio/x-pn-windows-acm"
	};
	int count;
	time_t max;

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 3);
	gtk_window_set_title(GTK_WINDOW(window), _("Preferences"));
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
	icon = gdk_pixbuf_new_from_file(HAPPYTIMER_ICON, NULL);
	gtk_window_set_icon(GTK_WINDOW(window), icon);
	gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);
	gdk_pixbuf_unref(icon);

	g_signal_connect(G_OBJECT(window), "delete_event",
			G_CALLBACK(show_preference), (gpointer)TRUE);

	basic_vbox = gtk_vbox_new(FALSE, 5);
	gtk_widget_show(basic_vbox);
	gtk_container_add(GTK_CONTAINER(window), basic_vbox);
	
	notebook = gtk_notebook_new();
	gtk_widget_show(notebook);
	gtk_box_pack_start(GTK_BOX(basic_vbox), notebook, TRUE, TRUE, 0);

	vbox = gtk_vbox_new(FALSE, 15);
	gtk_widget_show(vbox);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

	label = gtk_label_new(_("Defaults"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);

	rTSound_checkbox = gtk_check_button_new_with_label(_("Alerts when exact time"));
	gtk_box_pack_start(GTK_BOX(vbox), rTSound_checkbox, FALSE, FALSE, 0);
	gtk_widget_show(rTSound_checkbox);

	if (!strcmp(prefer.rtsound, "on")) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rTSound_checkbox),
				TRUE);
	}

	g_signal_connect(G_OBJECT(rTSound_checkbox), "toggled",
			G_CALLBACK(on_sound_checkbox_clicked), (gpointer)"rtsound");

	tSound_checkbox = gtk_check_button_new_with_label(_("Alerts when timer become 0"));
	gtk_box_pack_start(GTK_BOX(vbox), tSound_checkbox, FALSE, FALSE, 0);
	gtk_widget_show(tSound_checkbox);

	if (!strcmp(prefer.tsound, "on")) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tSound_checkbox), TRUE);
	}

	g_signal_connect(G_OBJECT(tSound_checkbox), "toggled",
			G_CALLBACK(on_sound_checkbox_clicked), (gpointer)"tsound");

	pTimer_checkbox = gtk_check_button_new_with_label(_("Start with previous timer settings.."));
	gtk_box_pack_start(GTK_BOX(vbox), pTimer_checkbox, FALSE, FALSE, 0);
	gtk_widget_show(pTimer_checkbox);

	if (!strcmp(prefer.load_ptime, "on")) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pTimer_checkbox), TRUE);
	}

	g_signal_connect(G_OBJECT(pTimer_checkbox), "toggled",
			G_CALLBACK(on_sound_checkbox_clicked), (gpointer)"load_ptime");

	frame = gtk_frame_new(_("Choose the maximum amount of time (1 ~ 23hrs)"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_widget_show(hbox);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 3);
	gtk_container_add(GTK_CONTAINER(frame), hbox);

	max = atol(prefer.maxtimer);
	maxtimer_spin_adj = (GtkAdjustment *)gtk_adjustment_new((max/60/60), 1, 23, 1, 10, 0);
	maxtimer_spin = gtk_spin_button_new(maxtimer_spin_adj, 1, 0);
	gtk_widget_show(maxtimer_spin);
	gtk_box_pack_start(GTK_BOX(hbox), maxtimer_spin, TRUE, TRUE, 0);
	gtk_widget_set_tooltip_text(maxtimer_spin, _("Will be applied when the timer does not work."));
	g_signal_connect(G_OBJECT(maxtimer_spin), "value-changed",
			G_CALLBACK(on_maxtimer_spin_changed), NULL);

	label = gtk_label_new(_("hr"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	frame = gtk_frame_new(_("Choose your hourly signal sounds"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);	

	hbox = gtk_hbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 3);
	gtk_widget_show(hbox);
	gtk_container_add(GTK_CONTAINER(frame), hbox);

	sound_combobox = gtk_combo_box_new_text();
	gtk_widget_show(sound_combobox);
	gtk_box_pack_start(GTK_BOX(hbox), sound_combobox, TRUE, TRUE, 0);

	/* /usr/share/happytimer/sound 디렉토리를 검색해서
	존재하는 음성 일람을 콤보박스에 등록합니다. */
	append_sound_to_combo(sound_combobox);

	g_signal_connect(G_OBJECT(sound_combobox), "changed",
			G_CALLBACK(on_sound_combobox_changed), NULL);

	listenSound_button = gtk_button_new();
	gtk_widget_show(listenSound_button);
	gtk_box_pack_start(GTK_BOX(hbox), listenSound_button, FALSE, FALSE, 0);

	g_signal_connect(G_OBJECT(listenSound_button), "clicked",
			G_CALLBACK(on_listenSound_button_clicked), NULL);

	image = gtk_image_new_from_stock("gtk-media-play", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show(image);
	gtk_container_add(GTK_CONTAINER(listenSound_button), image);

	frame = gtk_frame_new(_("Choose your timer sounds"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	audio_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(audio_filter, _("Audio Files"));
	for (count = 0; count < G_N_ELEMENTS(audio_mime_types); count++)
		gtk_file_filter_add_mime_type(audio_filter, audio_mime_types[count]);

	all_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(all_filter, _("All files"));
	gtk_file_filter_add_pattern(all_filter, "*");

	sound_chooser = gtk_file_chooser_button_new(_("sound to play"),
				GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_container_add(GTK_CONTAINER(frame), sound_chooser);
	gtk_container_set_border_width(GTK_CONTAINER(sound_chooser), 3);
	gtk_widget_show(sound_chooser);

	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(sound_chooser), audio_filter);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(sound_chooser), all_filter);

	g_signal_connect(G_OBJECT(sound_chooser), "selection-changed",
			G_CALLBACK(sound_changed), NULL);
	g_signal_connect(G_OBJECT(sound_chooser), "update-preview",
			G_CALLBACK(sound_preview), NULL);

	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(sound_chooser), prefer.tsoundfile);

	frame = gtk_frame_new(_("Select Timer's Background Image"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	image_button = gtk_button_new();
	gtk_widget_show(image_button);
	gtk_container_set_border_width(GTK_CONTAINER(image_button), 3);
	gtk_container_add(GTK_CONTAINER(frame), image_button);
	g_signal_connect(G_OBJECT(image_button), "clicked",
		G_CALLBACK(on_image_button_clicked), NULL);

	pixbuf = gdk_pixbuf_new_from_file_at_scale(prefer.timagefile,
		-1, 100,
		TRUE,
		NULL);

	image = gtk_image_new_from_pixbuf(pixbuf);
	gdk_pixbuf_unref(pixbuf);
	gtk_button_set_image(GTK_BUTTON(image_button), image);

/***********************************************************/

	vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
	gtk_widget_show(vbox);

	label = gtk_label_new(_("Etc..."));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);

	frame = gtk_frame_new(_("Set in preset"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);

	vbox2 = gtk_vbox_new(FALSE, 5);
	gtk_widget_show(vbox2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox2), 3);
	gtk_container_add(GTK_CONTAINER(frame), vbox2);

	pp_combobox = gtk_combo_box_new_text();
	gtk_widget_show(pp_combobox);
	gtk_box_pack_start(GTK_BOX(vbox2), pp_combobox, TRUE, TRUE, 0);

	add_preset_to_pp(pp_combobox);

	hbox = gtk_hbox_new(FALSE, 3);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 0);

	ppRemove_button = gtk_button_new_from_stock("gtk-remove");
	gtk_widget_show(ppRemove_button);
	gtk_box_pack_end(GTK_BOX(hbox), ppRemove_button, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(ppRemove_button), "clicked",
				G_CALLBACK(on_ppRemove_button_clicked), (gpointer)pp_combobox);

	ppEdit_button = gtk_button_new_from_stock("gtk-edit");
	gtk_widget_show(ppEdit_button);
	gtk_box_pack_end(GTK_BOX(hbox), ppEdit_button, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(ppEdit_button), "clicked",
				G_CALLBACK(on_ppEdit_button_clicked), (gpointer)pp_combobox);

	ppAdd_button = gtk_button_new_from_stock("gtk-add");
	gtk_widget_show(ppAdd_button);
	gtk_box_pack_end(GTK_BOX(hbox), ppAdd_button, FALSE, FALSE, 0);
	g_signal_connect(G_OBJECT(ppAdd_button), "clicked",
				G_CALLBACK(on_ppAdd_button_clicked), (gpointer)pp_combobox);
/***********************************************************/

	hseparator = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(basic_vbox), hseparator, FALSE, FALSE, 0);
	gtk_widget_show(hseparator);

	hbuttonbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(basic_vbox), hbuttonbox, FALSE, FALSE, 0);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbuttonbox), GTK_BUTTONBOX_END);
	gtk_widget_show(hbuttonbox);

	closeButton = gtk_button_new_from_stock("gtk-close");
	gtk_container_add(GTK_CONTAINER(hbuttonbox), closeButton);
	gtk_widget_show(closeButton);

	g_signal_connect(G_OBJECT(closeButton), "clicked",
			G_CALLBACK(show_preference), (gpointer)TRUE);

	return window;
}

static void display_about_dialog(void)
{
	GdkPixbuf *logo;
	GError *error = NULL;
	static GtkWidget *about = NULL;
	const gchar *authors[] = {_("Kim Jin Chul (shriekout@gmail.com)"), NULL};
	const gchar *artists[] = {_("Kim Jin Chul (shriekout@gmail.com)"), _("KangBunDo (bundo@bundo.biz)"),NULL};
	const gchar *translator_credits = _("Young-chil-e (In English translation, 2008.11.26)\nYoung-chil-e (In Philippines (Tagalog) translation, 2008.11.29)");
	const gchar *website = "http://happytimer.kldp.net";
	const gchar *website_label = _("Homepage (http://happytimer.kldp.net)");
	const gchar *license = _("This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; eitherversion 2 of the License, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public License for more details.\n\nYou should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.");

	if (about != NULL) {
		gtk_window_present(GTK_WINDOW(about));
		return;
	}

	logo = gdk_pixbuf_new_from_file(HAPPYTIMER_ICON, &error);

	about = gtk_about_dialog_new();
	gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), "Happytimer");
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), _("Copyright (C) 2007- Kim Jin Chul"));
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), _("Happytimer is a timer application that\nsits in GNOME's notification area.\nYou will never forget\nyour happiest time anymore."));
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about), website);
	gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(about), website_label);
	gtk_about_dialog_set_wrap_license(GTK_ABOUT_DIALOG(about), TRUE);
	gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about), license);
	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about), authors);
	gtk_about_dialog_set_artists(GTK_ABOUT_DIALOG(about), artists);
	gtk_about_dialog_set_translator_credits(GTK_ABOUT_DIALOG(about), translator_credits);
	gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about), logo);
	gdk_pixbuf_unref(logo);

	gtk_window_set_icon_from_file(GTK_WINDOW(about), HAPPYTIMER_ICON, NULL);

	gtk_dialog_run(GTK_DIALOG(about));
	gtk_widget_destroy(about);
	about = NULL;
}

/* 알람 종료시 나타날 이미지 파일 선택 다이얼로그 */
GtkWidget *create_imagechooserdialog(void)
{
	GtkWidget *imagechooserdialog;
	GtkWidget *vbox;
	GdkPixbuf *pixbuf;
	GtkWidget *preview_image;
	GtkFileFilter *gdkimage_filter, *all_filter;
	
	imagechooserdialog = gtk_file_chooser_dialog_new(_("Choose image"),
		NULL,
		GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_OPEN, GTK_RESPONSE_OK,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		NULL);
	g_object_set(imagechooserdialog, "show-hidden", TRUE, NULL);
	gtk_window_set_type_hint(GTK_WINDOW(imagechooserdialog), GDK_WINDOW_TYPE_HINT_DIALOG);
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(imagechooserdialog), prefer.timagefile);

	gdkimage_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(gdkimage_filter, _("Image File"));
	gtk_file_filter_add_pixbuf_formats(gdkimage_filter);
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(imagechooserdialog), gdkimage_filter);

	all_filter = gtk_file_filter_new();
	gtk_file_filter_set_name(all_filter, _("All files"));
	gtk_file_filter_add_pattern(all_filter, "*");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(imagechooserdialog), all_filter);

	vbox = GTK_DIALOG(imagechooserdialog)->vbox;
	gtk_widget_show(vbox);

	pixbuf = gdk_pixbuf_new_from_file_at_scale(prefer.timagefile,
		-1, 100,
		TRUE,
		NULL);

	if (!pixbuf) {
		preview_image = gtk_image_new_from_icon_name("gtk-missing-image", GTK_ICON_SIZE_DIALOG);
		gtk_image_set_pixel_size(GTK_IMAGE(preview_image), 100);
	} else {
		preview_image = gtk_image_new_from_pixbuf(pixbuf);
		gdk_pixbuf_unref(pixbuf);
	}

	gtk_box_pack_start(GTK_BOX(vbox), preview_image, TRUE, TRUE, 0);
	gtk_widget_show(preview_image);

	gtk_widget_show(imagechooserdialog);

	g_signal_connect(G_OBJECT(imagechooserdialog), "update-preview",
			G_CALLBACK(image_preview), (gpointer)preview_image);
	
	return imagechooserdialog;
}

GtkWidget *create_extension_box(void)
{
	GtkWidget *extension_vbox;
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkAdjustment *hr_adj;
	GtkWidget *hr_spin;
	GtkAdjustment *min_adj;
	GtkWidget *min_spin;
	GtkAdjustment *sec_adj;
	GtkWidget *sec_spin;
	GtkWidget *preset_combobox;
	GtkWidget *repeatTimer_checkbox;

	extension_vbox = gtk_vbox_new(FALSE, 3);

	frame = gtk_frame_new(_("Set the timer details"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(extension_vbox), frame, TRUE, TRUE, 0);

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_widget_show(hbox);
	gtk_container_add(GTK_CONTAINER(frame), hbox);

	hr_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 23, 1, 10, 0);
	hr_spin = gtk_spin_button_new(hr_adj, 1, 0);
	gtk_widget_show(hr_spin);
	gtk_box_pack_start(GTK_BOX(hbox), hr_spin, FALSE, FALSE, 2);
	g_signal_connect(G_OBJECT(hr_spin), "value-changed",
			G_CALLBACK(on_set_detail_spin_changed), "hour");
	g_object_set_data(G_OBJECT(extension_vbox), "hr_spin", (gpointer)hr_spin);

	label = gtk_label_new(_("hr"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

	min_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 59, 1, 10, 0);
	min_spin = gtk_spin_button_new(min_adj, 1, 0);
	gtk_widget_show(min_spin);
	gtk_box_pack_start(GTK_BOX(hbox), min_spin, FALSE, FALSE, 2);
	g_signal_connect(G_OBJECT(min_spin), "value-changed",
			G_CALLBACK(on_set_detail_spin_changed), "minute");
	g_object_set_data(G_OBJECT(extension_vbox), "min_spin", (gpointer)min_spin);

	label = gtk_label_new(_("min"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

	sec_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 59, 1, 10, 0);
	sec_spin = gtk_spin_button_new(sec_adj, 1, 0);
	gtk_widget_show(sec_spin);
	gtk_box_pack_start(GTK_BOX(hbox), sec_spin, FALSE, FALSE, 2);
	g_signal_connect(G_OBJECT(sec_spin), "value-changed",
			G_CALLBACK(on_set_detail_spin_changed), "second");
	g_object_set_data(G_OBJECT(extension_vbox), "sec_spin", (gpointer)sec_spin);

	label = gtk_label_new(_("sec"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

	preset_combobox = gtk_combo_box_new_text();
	gtk_widget_show(preset_combobox);
	gtk_box_pack_start(GTK_BOX(extension_vbox), preset_combobox, TRUE, TRUE, 2);
	g_object_set_data(G_OBJECT(extension_vbox), "preset_combobox", (gpointer)preset_combobox);
	
	append_preset_to_combo(preset_combobox);

	g_signal_connect(G_OBJECT(preset_combobox), "changed",
			G_CALLBACK(on_preset_combobox_changed), NULL);

	repeatTimer_checkbox = gtk_check_button_new_with_label(_("Repeat timer"));
	gtk_widget_set_tooltip_text(repeatTimer_checkbox, _("The image will not be shown."));
	gtk_widget_show(repeatTimer_checkbox);
	gtk_box_pack_start(GTK_BOX(extension_vbox), repeatTimer_checkbox, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(repeatTimer_checkbox), "toggled",
			G_CALLBACK(on_repeatTimer_checkbox_clicked), NULL);

	return extension_vbox;
}

GtkWidget *create_presetEAdialog(int what, struct presetList *sp)
{
	GtkWidget *dialog;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *entry;
	GtkWidget *label;
	GtkAdjustment *hr_adj;
	GtkWidget *hr_spin;
	GtkAdjustment *min_adj;
	GtkWidget *min_spin;
	GtkAdjustment *sec_adj;
	GtkWidget *sec_spin;
	int hr, min, sec;

	dialog = gtk_dialog_new_with_buttons(_("Set in preset"),
								NULL,
								GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
								GTK_STOCK_OK,
								GTK_RESPONSE_ACCEPT,
								GTK_STOCK_CANCEL,
								GTK_RESPONSE_REJECT,
								NULL);
	vbox = gtk_vbox_new(FALSE, 3);
	gtk_widget_show(vbox);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);

	entry = gtk_entry_new();
	gtk_widget_show(entry);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	g_object_set_data(G_OBJECT(dialog), "entry", (gpointer)entry);

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

	hr_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 23, 1, 10, 0);
	hr_spin = gtk_spin_button_new(hr_adj, 1, 0);
	gtk_widget_show(hr_spin);
	gtk_box_pack_start(GTK_BOX(hbox), hr_spin, FALSE, FALSE, 2);
	g_object_set_data(G_OBJECT(dialog), "hr_spin", (gpointer)hr_spin);

	label = gtk_label_new(_("hr"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

	min_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 59, 1, 10, 0);
	min_spin = gtk_spin_button_new(min_adj, 1, 0);
	gtk_widget_show(min_spin);
	gtk_box_pack_start(GTK_BOX(hbox), min_spin, FALSE, FALSE, 2);
	g_object_set_data(G_OBJECT(dialog), "min_spin", (gpointer)min_spin);

	label = gtk_label_new(_("min"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

	sec_adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 59, 1, 10, 0);
	sec_spin = gtk_spin_button_new(sec_adj, 1, 0);
	gtk_widget_show(sec_spin);
	gtk_box_pack_start(GTK_BOX(hbox), sec_spin, FALSE, FALSE, 2);
	g_object_set_data(G_OBJECT(dialog), "sec_spin", (gpointer)sec_spin);

	label = gtk_label_new(_("sec"));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);

	if (what == P_EDIT) {
		hr = (int)sp->timer / 3600;
		min = (int)sp->timer / 60 - hr * 60;
		sec = (int)sp->timer % 60;

		gtk_entry_set_text(GTK_ENTRY(entry), sp->name);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(hr_spin), hr);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(min_spin), min);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(sec_spin), sec);
	} else if (what == P_ADD) {
		gtk_entry_set_text(GTK_ENTRY(entry), _("happytimer"));
	}

	return dialog;
}

GtkWidget *create_presetremoveDialog(char *content)
{
	GtkWidget *dialog;

	dialog = gtk_message_dialog_new(NULL,
								GTK_DIALOG_DESTROY_WITH_PARENT,
								GTK_MESSAGE_QUESTION,
								GTK_BUTTONS_OK_CANCEL,
								_("Are you really want to remove this \"%s\"?"),
								content);

	return dialog;							
}
