/*
 *  @Copyright (C) 2007 Kim Jin Chul (shriekout@gmail.com).
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/************************************************
 * 시간 관련 함수들과 기타 함수들의 모음입니다.
 * **********************************************/

#include <time.h>
#include <gtk/gtk.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <libxml/parser.h>
#include <libintl.h>
#include "happytimer.h"
#include "callback.h"
#include "timecheck.h"
#include "interface.h"

static void countdown_function(void);

static time_t now;
static struct tm *ptr;
static int reti;

gboolean timeCheck(gpointer data)
{
	char *buf1, buf2[80];

	time(&now);

	ptr = localtime(&now);

	buf1 = (char *)malloc(7 * sizeof(char));

	if (!strcmp(varList.hour_clock, "12"))
		strftime(buf1, 7, "%I%M%S", ptr);
	else if (!strcmp(varList.hour_clock, "24"))
		strftime(buf1, 7, "%H%M%S", ptr);
	else if (!strcmp(varList.hour_clock, "1"))
		strftime(buf1, 7, "00%M%S", ptr);

	reti = regexec(&varList.mregex, buf1, 0, NULL, 0);

	if (!reti && (!strcmp(prefer.rtsound, "on"))) {
		buf1[2] = '\0';
		sprintf(buf2, "%s%sh.%s", prefer.rtsound_dir, buf1, varList.sf);

		gst_element_set_state(gstSound[RT_SND], GST_STATE_READY);
		gchar *uri = g_filename_to_uri(buf2, NULL, NULL);
		g_object_set(G_OBJECT(gstSound[RT_SND]), "uri", uri, NULL);
		gst_element_set_state(gstSound[RT_SND], GST_STATE_PLAYING);
		g_free(uri);
	}

	free(buf1);

	if ((varList.countdown == TRUE) && (varList.is_pause == FALSE)) {
		countdown_function();
	}

	return TRUE;
}

void set_timer(GtkAdjustment *adj)
{
	GtkWidget *tl;
	GtkWidget *ev;
	GtkWidget *hs;
	GtkWidget *ms;
	GtkWidget *ss;
	GtkWidget *ths;
	char *buf, buf2[80];
	time_t current_time;
	struct tm *ptr;

	current_time = (time_t)gtk_adjustment_get_value(adj);

	buf = return_strftime(current_time, FALSE);

	tl = g_object_get_data(G_OBJECT(timer), "timer_label");
	gtk_label_set_text(GTK_LABEL(tl), buf);
	sprintf(buf2, "Happytimer\n%s", buf);
	gtk_label_set_text(GTK_LABEL(tooltipTimer_label), buf2);
	free(buf);

	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");

	ptr = gmtime(&current_time);

	buf = (char *)malloc(sizeof(char) * 3);
	strftime(buf, 3, "%H", ptr);
	hs = g_object_get_data(G_OBJECT(ev), "hr_spin");
	varList.spin_hr = atoi(buf);
	free(buf);

	buf = (char *)malloc(sizeof(char) * 3);
	strftime(buf, 3, "%M", ptr);
	ms = g_object_get_data(G_OBJECT(ev), "min_spin");
	varList.spin_min = atoi(buf);
	free(buf);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(hs), varList.spin_hr);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ms), varList.spin_min);

	ths = g_object_get_data(G_OBJECT(timer), "timer_hscale");

	if (gtk_widget_is_focus(ths)) {
		ss = g_object_get_data(G_OBJECT(ev), "sec_spin");
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(ss), 0);
		varList.spin_sec = 0;
	}
}


char *return_strftime(time_t sec, gboolean returnSec)
{
	struct tm *ptr;
	char buf[80], *mbuf;

	ptr = gmtime(&sec);

	if (returnSec == TRUE) {
		strftime(buf, 79, _("%Hhr %Mmin %Ssec"), ptr);
	} else {
		strftime(buf, 79, _("%Hhr %Mmin 00sec"), ptr);
	}

	mbuf = (char *)malloc((strlen(buf) + 1) * sizeof(char));
	strcpy(mbuf, buf);

	return mbuf;
}

static void save_content_xml(xmlNodePtr node, gpointer data)
{
	xmlNodePtr cur_node = NULL;
	char *buf;

	for (cur_node = node; cur_node; cur_node = cur_node->next) {
		if (cur_node->type == XML_ELEMENT_NODE) {
			if (!strcmp((char *) cur_node->name, (char *) data)) {
				if (!strcmp((char *) cur_node->name, "load_ptime")) {
					buf = strdup(prefer.load_ptime);
				} else if (!strcmp((char *) cur_node->name, "rtsound")) {
					puts("abc");
					buf = strdup(prefer.rtsound);
				} else if (!strcmp((char *) cur_node->name, "tsound")) {
					buf = strdup(prefer.tsound);
				} else if (!strcmp((char *) cur_node->name, "maxtimer")) {
					buf = strdup(prefer.maxtimer);
				} else if (!strcmp((char *) cur_node->name, "rtsound_dir")) {
					buf = strdup(prefer.rtsound_dir);
				} else if (!strcmp((char *) cur_node->name, "timagefile")) {
					buf = strdup(prefer.timagefile);
				} else if (!strcmp((char *) cur_node->name, "tsoundfile")) {
					buf = strdup(prefer.tsoundfile);
				}

				xmlNodeSetContent(cur_node, BAD_CAST buf);
				free(buf);
			}
		}
	save_content_xml(cur_node->children, data);
	}
}

void save_preference(GtkWidget *widget, gpointer data)
{
	xmlDocPtr doc;
	xmlNodePtr root_node;

	doc = xmlParseFile(prefer.prefer_file);

	root_node = xmlDocGetRootElement(doc);
	save_content_xml(root_node, data);

	xmlSaveFormatFileEnc(prefer.prefer_file, doc, "UTF-8", 1);
	xmlFreeDoc(doc);
}

static void countdown_function(void)
{
	GtkWidget *tl;
	GtkWidget *tev;
	char *buf, buf2[80];

	if (varList.timerCount > 0)
		varList.timerCount--;

	buf = return_strftime(varList.timerCount, TRUE);

	tl = g_object_get_data(G_OBJECT(timer), "timer_label");
	gtk_label_set_text(GTK_LABEL(tl), buf);
	sprintf(buf2, _("Happytimer\n%s"), buf);
	gtk_label_set_text(GTK_LABEL(tooltipTimer_label), buf2);
	free(buf);

	if (varList.timerCount == 600) {
		tev = g_object_get_data(G_OBJECT(trayicon), "trayiconEbox");
		on_trayiconEbox_enter_notify_event(tev, NULL, (gpointer)"EVENT");
	}

	if (varList.timerCount == 0 || varList.timerCount < 0) {
		if (!varList.repeat)
			alarm_window = create_alarm_window();

		if (!strcmp(prefer.tsound, "on") && prefer.tsoundfile) {
			gst_element_set_state(gstSound[ARM_SND], GST_STATE_READY);
			gchar *uri = g_filename_to_uri(prefer.tsoundfile, NULL, NULL);
			g_object_set(G_OBJECT(gstSound[ARM_SND]), "uri", uri, NULL);
			gst_element_set_state(gstSound[ARM_SND], GST_STATE_PLAYING);
			g_free(uri);
		}

		timerGo(NULL, "END");
	}
}

/* 환경설정 정각알림 콤보박스에 사용 가능한 정각알림소리를 등록합니다. */
void append_sound_to_combo(GtkWidget *widget)
{
	FILE *fp;
	char buf[80], fullname[256];
	DIR *dp;
	struct dirent *ep;
	char *np;
	struct stat statbuf;
	int count = 0;
	struct soundDir *new, *cur;

	soundDir_head = NULL;
	new = NULL;
	cur = NULL;

	new = (struct soundDir *)malloc(sizeof(struct soundDir));
	new->next = NULL;
	new->dirname = strdup(HAPPYTIMER_SOUND);
	soundDir_head = new;

	sprintf(fullname, "%s%s", HAPPYTIMER_SOUND, VOICE_INFO);

	if ((fp = fopen(fullname, "r")) == NULL) {
		fprintf(stderr, "Error opening file %s.\n", fullname);
		exit(1);
	}
	
	fgets(buf, 79, fp);
	buf[strlen(buf) - 1] = '\0';
	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), buf);

	if (!strcmp(prefer.rtsound_dir, new->dirname))
		gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);
	
	fclose(fp);

	count++;
	
	if ((dp = opendir(HAPPYTIMER_SOUND)) == NULL) {
		fprintf(stderr, "Error opening directory %s.\n", HAPPYTIMER_SOUND);
		exit(1);
	}

	cur = soundDir_head;

	while (1) {
		ep = readdir(dp);

		if (ep == NULL)
			break;
		np = ep->d_name;

		if ((!strcmp(np, ".")) || (!strcmp(np, "..")))
			continue;

		sprintf(fullname, "%s%s", HAPPYTIMER_SOUND, np);

		stat(fullname, &statbuf);

		if (S_ISDIR(statbuf.st_mode)) {
			sprintf(fullname, "%s/", fullname);

			new = (struct soundDir *)malloc(sizeof(struct soundDir));
			new->dirname = strdup(fullname);
			cur->next = new;
			new->next = NULL;
			cur = new;

			sprintf(fullname, "%s%s", fullname, VOICE_INFO);

			if ((fp = fopen(fullname, "r")) == NULL) {
				fprintf(stderr, "Error opening file %s.\n", fullname);
				exit(1);
			}
			
			fgets(buf, 79, fp);
			buf[strlen(buf) - 1] = '\0';
			
			fclose(fp);
			gtk_combo_box_append_text(GTK_COMBO_BOX(widget), buf);

			if (!strcmp(prefer.rtsound_dir, cur->dirname))
				gtk_combo_box_set_active(GTK_COMBO_BOX(widget), count);

			count++;
		}	
	}

	closedir(dp);
}

/* 타이머 최대설정 시간이 변경되었을 때 */
void maxtimer_changed(void)
{
	GtkAdjustment *ta;
	GtkWidget *tl;
	GtkWidget *ml;
	GtkWidget *ev;
	GtkWidget *ss;
	time_t max, current;
	char *buf, buf2[80];
	long get_time, spin_sec, sec;
	struct tm *ptr;

	ta = g_object_get_data(G_OBJECT(timer), "timer_adj");
	tl = g_object_get_data(G_OBJECT(timer), "timer_label");
	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
	ss = g_object_get_data(G_OBJECT(ev), "sec_spin");

	max = atol(prefer.maxtimer);
	current = gtk_adjustment_get_value(ta);

	if (current > max) {
		gtk_adjustment_set_value(ta, max);
	}

	ta->upper = max;

	ml = g_object_get_data(G_OBJECT(timer), "max_label");

	buf = (char *)malloc(sizeof(char) * 10);
	sprintf(buf, _("%ldhours"), max/3600);
	gtk_label_set_text(GTK_LABEL(ml), buf);
	free(buf);

	/* 타이머 종료시 타이머 레벨을 처음 셋팅된 시간으로 변경한다 */
	get_time = (long)gtk_adjustment_get_value(ta);
	ptr = gmtime(&get_time);
	buf = (char*)malloc(3 * sizeof(char));
	strftime(buf, 3, "%S", ptr);
	sec = atol(buf);
	free(buf);

	spin_sec = (long)gtk_spin_button_get_value(GTK_SPIN_BUTTON(ss));
	get_time = get_time - sec + spin_sec;

	buf = return_strftime(get_time, TRUE);
	gtk_label_set_text(GTK_LABEL(tl), buf);
	sprintf(buf2, _("Happytimer\n%s"), buf);
	gtk_label_set_text(GTK_LABEL(tooltipTimer_label), buf2);
	free(buf);
}

void timerWidget_sensitive(gboolean sensitive)
{
	GtkWidget *th, *mb, *pb, *ev, *hs, *ms, *ss, *pc;

	th = g_object_get_data(G_OBJECT(timer), "timer_hscale");
	mb = g_object_get_data(G_OBJECT(timer), "minus_button");
	pb = g_object_get_data(G_OBJECT(timer), "plus_button");
	ev = g_object_get_data(G_OBJECT(timer), "extension_vbox");
	hs = g_object_get_data(G_OBJECT(ev), "hr_spin");
	ms = g_object_get_data(G_OBJECT(ev), "min_spin");
	ss = g_object_get_data(G_OBJECT(ev), "sec_spin");
	pc = g_object_get_data(G_OBJECT(ev), "preset_combobox");

	gtk_widget_set_sensitive(th, sensitive);
	gtk_widget_set_sensitive(mb, sensitive);
	gtk_widget_set_sensitive(pb, sensitive);
	gtk_widget_set_sensitive(hs, sensitive);
	gtk_widget_set_sensitive(ms, sensitive);
	gtk_widget_set_sensitive(ss, sensitive);
	gtk_widget_set_sensitive(pc, sensitive);
}

static void read_preset_xml(xmlNodePtr node, GtkWidget *widget)
{
	xmlNodePtr cur_node = NULL;
	xmlChar *name, *timer;
	char *buf1, buf2[256];
	struct presetList *new, *cur;

	for (cur_node = node; cur_node; cur_node = cur_node->next) {
		if (cur_node->type == XML_ELEMENT_NODE) {
			if (!strcmp((char *) cur_node->name, "preset")) {
				name = xmlGetProp(cur_node, BAD_CAST "name");
				timer = xmlGetProp(cur_node, BAD_CAST "timer");

				buf1 = return_strftime(atol((char *)timer), TRUE);
				sprintf(buf2, "%s (%s)", name, buf1);

				cur = presetList_head;

				while (cur->next != NULL)
					cur = cur->next;

				new = (struct presetList *)malloc(sizeof(struct presetList));
				cur->next = new;
				new->next = NULL;
				cur = new;

				cur->name = (char *)name;
				cur->timer = atol((char *)timer);

				gtk_combo_box_append_text(GTK_COMBO_BOX(widget), buf2);

				free(timer);
				free(buf1);
			}
		}
	read_preset_xml(cur_node->children, widget);
	}
}

void append_preset_to_combo(GtkWidget *widget)
{
	xmlDocPtr doc;
	xmlNodePtr root_node;
	struct presetList *new;

	presetList_head = NULL;
	new = NULL;

	new = (struct presetList *)malloc(sizeof(struct presetList));
	presetList_head = new;
	new->next = NULL;
	new->name = strdup(_("Preset"));
	new->timer = 0;

	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), new->name);
	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);

	doc =  xmlReadFile(prefer.prefer_file, NULL, 0);

	root_node = xmlDocGetRootElement(doc);
	read_preset_xml(root_node, widget);

	xmlFreeDoc(doc);
}

void add_preset_to_pp(GtkWidget *widget)
{
	char *buf1, buf2[80];

	struct presetList *ptr;

	ptr = presetList_head->next;

	while (ptr != NULL) {
		buf1 = return_strftime(ptr->timer, TRUE);
		sprintf(buf2, "%s (%s)", ptr->name, buf1);

		gtk_combo_box_append_text(GTK_COMBO_BOX(widget), buf2);

		free(buf1);

		ptr = ptr->next;
	}

	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);
}

static void REA_preset_xml(xmlNodePtr node, struct presetList *sp, struct presetList *sp2, int what)
{
	xmlNodePtr cur_node, child_node;
	xmlChar *name, *timer;
	char buf[80];

	for (cur_node = node; cur_node; cur_node = cur_node->next) {
		if (cur_node->type == XML_ELEMENT_NODE) {
			if (what != P_ADD) {
				if (!strcmp((char *) cur_node->name, "preset")) {
					name = xmlGetProp(cur_node, BAD_CAST "name");
					timer = xmlGetProp(cur_node, BAD_CAST "timer");

					if (!strcmp((char *)name, sp->name) && (atol((char *)timer) == sp->timer)) {
						if (what == P_REMOVE) {
							xmlUnlinkNode(cur_node);
							xmlFreeNode(cur_node);
						} else if (what == P_EDIT) {
							sprintf(buf, "%ld", sp2->timer);
							xmlSetProp(cur_node, BAD_CAST "name", BAD_CAST sp2->name);
							xmlSetProp(cur_node, BAD_CAST "timer", BAD_CAST buf);
						}
					}

					free(name);
					free(timer);
				}
			} else {
				if (!strcmp((char *) cur_node->name, "preset_group")) {
					sprintf(buf, "%ld", sp->timer);
					child_node = xmlNewChild (cur_node, NULL, BAD_CAST "preset", NULL);
					xmlNewProp(child_node, BAD_CAST "name", BAD_CAST sp->name);
					xmlNewProp(child_node, BAD_CAST "timer", BAD_CAST buf);
				}
			}
		}
		REA_preset_xml(cur_node->children, sp, sp2, what);
	}	
}

void ppREA(struct presetList *sp, struct presetList *sp2, int what)
{
	xmlDocPtr doc;
	xmlNodePtr root_node;

	doc = xmlParseFile(prefer.prefer_file);

	root_node = xmlDocGetRootElement(doc);
	REA_preset_xml(root_node, sp, sp2, what);

	xmlSaveFormatFileEnc(prefer.prefer_file, doc, "UTF-8", 1);
	xmlFreeDoc(doc);
}

/* xml을 검색하여, 검색된 콘덴츠를 리턴합니다. 이 함수 사용후 xmlFree 함수로 메모리를 해제시켜줘야 합니다. */
xmlNodePtr search_xml_name(xmlNodePtr node, const char *name)
{
	xmlNodePtr cur_node = NULL;
	xmlNodePtr ret = NULL;

	for (cur_node = node; cur_node; cur_node = cur_node->next) {
		if (cur_node->type == XML_ELEMENT_NODE) {
			if (!strcmp((char *)cur_node->name, name)) {
				ret = (xmlNodePtr) xmlMalloc(sizeof(xmlNode));
				memmove((void *)ret, (void *)cur_node, sizeof(xmlNode));
				return ret;
			}
		}

		if (ret) {
			return ret;
		} else {
			ret = search_xml_name(cur_node->children, name);
		}
	}

	return ret;
}

struct hms_struct return_hms_struct(long t)
{
	struct tm *ptr;
	struct hms_struct tt;
	char buf[3];

	ptr = gmtime(&t);

	strftime(buf, 3, "%H", ptr);
	tt.hr = atoi(buf);

	strftime(buf, 3, "%M", ptr);
	tt.min = atoi(buf);

	strftime(buf, 3, "%S", ptr);
	tt.sec = atoi(buf);

	return tt;	
}
